@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 13:34:47 1995
  * Project:     INN (innfeed)
  * File:        connection.c
- * RCSId:       $Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp $
+ * RCSId:       $Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp brister $
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
  *              Permission to use, copy, modify, and distribute this
@@ -78,7 +78,7 @@
   */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp $" ;
+static const char *rcsid = "$Id: connection.c,v 1.21 1996/06/05 14:00:48 brister Exp brister $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -205,6 +205,7 @@ struct connection_s
     TimeoutId sleepTimerId ;
 
 
+    bool loggedNoCr ;           /* true if we logged the NOCR_MSG */
     bool immedRecon ;           /* true if we recon immediately after flushing. */
     bool doesStreaming ;        /* true if remote will handle streaming */
     bool quitWasIssued ;          /* true if QUIT command was sent. */
@@ -267,6 +268,8 @@ static void articleTimeoutCbk (TimeoutId id, void *data) ;
 static void cxnWorkProc (EndPoint ep, void *data) ;
 
 
+static void cxnSleepOrDie (Connection cxn) ;
+
 /* Response processing. */
 static void processResponse205 (Connection cxn, char *response) ;
 static void processResponse238 (Connection cxn, char *response) ;
@@ -316,7 +319,7 @@ static bool remArtHolder (ArtHolder art, ArtHolder *head, u_int *count) ;
 static void appendArtHolder (ArtHolder artH, ArtHolder *head, u_int *count) ;
 static ArtHolder artHolderByMsgId (const char *msgid, ArtHolder head) ;
 
-
+static int fudgeFactor (int initVal) ;
 
 
 
@@ -396,13 +399,13 @@ Connection newConnection (Host host,
   cxn->ipName = strdup (ipname) ;
   cxn->port = portNum ;
 
-  cxn->articleReceiptTimeout = articleReceiptTimeout ;
+  cxn->articleReceiptTimeout = fudgeFactor (articleReceiptTimeout) ;
   cxn->artReceiptTimerId = 0 ;
 
-  cxn->readTimeout = respTimeout ;
+  cxn->readTimeout = fudgeFactor (respTimeout) ;
   cxn->readBlockedTimerId = 0 ;
 
-  cxn->flushTimeout = flushTimeout ;
+  cxn->flushTimeout = fudgeFactor (flushTimeout) ;
   cxn->flushTimerId = 0 ;
 
   cxn->onThreshold = lowPassHigh ;
@@ -1178,7 +1181,7 @@ static void getBanner (EndPoint e, IoStatus i, Buffer *b, void *d)
     {
       trim_ws (p) ;
 
-      syslog (LOG_ERR, RESPONSE_BAD_FORMAT, peerName, cxn->ident, p) ;
+      syslog (LOG_ERR, INVALID_RESP_FORMAT, peerName, cxn->ident, p) ;
 
       cxnSleep (cxn) ;
     }
@@ -1469,8 +1472,12 @@ static void responseIsRead (EndPoint e, IoStatus i, Buffer *b, void *d)
       if (*endr != '\r')
         endr++ ;
 
-      if (next - endr != 2)     /* only a newline there. we'll live with it */
-        syslog (LOG_ERR, NOCR_MSG, peerName, cxn->ident) ;
+      if (next - endr != 2 && !cxn->loggedNoCr)
+        {
+          /* only a newline there. we'll live with it */
+          syslog (LOG_ERR, NOCR_MSG, peerName, cxn->ident) ;
+          cxn->loggedNoCr = true ;
+        }
 
       *endr = '\0' ;
 
@@ -1562,7 +1569,7 @@ static void responseIsRead (EndPoint e, IoStatus i, Buffer *b, void *d)
           default:
             syslog (LOG_ERR, UNKNOWN_RESPONSE, peerName, cxn->ident,
                     code, response) ;
-            cxnSleep (cxn) ;
+            cxnSleepOrDie (cxn) ;
             break ;
         }
 
@@ -1963,6 +1970,19 @@ static void cxnWorkProc (EndPoint ep, void *data)
  *
  ***************************************************************************/
 
+
+/*
+ * A connection needs to sleep, but if it's closing it needs to die instead.
+ */
+static void cxnSleepOrDie (Connection cxn)
+{
+  if (cxn->state == cxnClosingS)
+    cxnDead (cxn) ;
+  else
+    cxnSleep (cxn) ;
+}
+
+
 /*
  * Handle the response 205 to our QUIT command, which means the
  * remote is going away and we can happily cleanup
@@ -2013,8 +2033,8 @@ static void processResponse205 (Connection cxn, char *response)
 
       case cxnFeedingS:
         /* this shouldn't ever happen... */
-        syslog (LOG_NOTICE,CXN_DROPPING,hostPeerName (cxn->myHost),
-                cxn->ident) ;
+        syslog (LOG_NOTICE,BAD_RESPONSE,hostPeerName (cxn->myHost),
+                cxn->ident, 205) ;
         cxnSleep (cxn) ;
         break ;
 
@@ -2044,8 +2064,14 @@ static void processResponse238 (Connection cxn, char *response)
 
   msgid = getMsgId (response) ;
 
-  if (msgid == NULL || strlen (msgid) == 0 ||
-      (artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL)
+  if (cxn->checkRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,238) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else if (msgid == NULL || strlen (msgid) == 0 ||
+           (artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL)
     noSuchMessageId (cxn,238,msgid,response) ;
   else
     {
@@ -2080,8 +2106,14 @@ static void processResponse431 (Connection cxn, char *response)
 
   msgid = getMsgId (response) ;
 
-  if ((msgid == NULL || strlen (msgid) == 0) ||
-      ((artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL))
+  if (cxn->checkRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,431) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else if (msgid == NULL || strlen (msgid) == 0 ||
+           (artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL)
     noSuchMessageId (cxn,431,msgid,response) ;
   else
     {
@@ -2115,8 +2147,14 @@ static void processResponse438 (Connection cxn, char *response)
 
   msgid = getMsgId (response) ;
 
-  if ((msgid == NULL || strlen (msgid) == 0) ||
-      ((artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL))
+  if (cxn->checkRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,438) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else if (msgid == NULL || strlen (msgid) == 0 ||
+           (artHolder = artHolderByMsgId (msgid, cxn->checkRespHead)) == NULL)
     noSuchMessageId (cxn,438,msgid,response) ;
   else
     {
@@ -2151,8 +2189,14 @@ static void processResponse239 (Connection cxn, char *response)
 
   msgid = getMsgId (response) ;
 
-  if ((msgid == NULL || strlen (msgid) == 0) ||
-      ((artHolder = artHolderByMsgId (msgid, cxn->takeRespHead)) == NULL))
+  if (cxn->takeRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,239) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else if (msgid == NULL || strlen (msgid) == 0 ||
+           (artHolder = artHolderByMsgId (msgid, cxn->takeRespHead)) == NULL)
     noSuchMessageId (cxn,239,msgid,response) ;
   else
     {
@@ -2189,8 +2233,14 @@ static void processResponse439 (Connection cxn, char *response)
 
   msgid = getMsgId (response) ;
 
-  if ((msgid == NULL || strlen (msgid) == 0) ||
-      ((artHolder = artHolderByMsgId (msgid, cxn->takeRespHead)) == NULL))
+  if (cxn->takeRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,239) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else if (msgid == NULL || strlen (msgid) == 0 ||
+           (artHolder = artHolderByMsgId (msgid, cxn->takeRespHead)) == NULL)
     noSuchMessageId (cxn,439,msgid,response) ;
   else
     {
@@ -2227,16 +2277,25 @@ static void processResponse235 (Connection cxn, char *response)
   ASSERT (cxn->takeRespHead != NULL) ;
   VALIDATE_CONNECTION (cxn) ;
 
-  /* now remove the article from the queue and tell the Host to forget
-     about it. */
-  artHolder = cxn->takeRespHead ;
-
-  cxn->takeRespHead = NULL ;
-  cxn->articleQTotal = 0 ;
-  cxn->takesOkayed++ ;
-
-  hostArticleAccepted (cxn->myHost, cxn, artHolder->article) ;
-  delArtHolder (artHolder) ;
+  if (cxn->takeRespHead == NULL) /* peer is confused */
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,235) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else
+    {
+      /* now remove the article from the queue and tell the Host to forget
+         about it. */
+      artHolder = cxn->takeRespHead ;
+      
+      cxn->takeRespHead = NULL ;
+      cxn->articleQTotal = 0 ;
+      cxn->takesOkayed++ ;
+      
+      hostArticleAccepted (cxn->myHost, cxn, artHolder->article) ;
+      delArtHolder (artHolder) ;
+    }
 }
 
 
@@ -2258,11 +2317,20 @@ static void processResponse335 (Connection cxn, char *response)
   ASSERT (cxn->checkRespHead != NULL) ;
   VALIDATE_CONNECTION (cxn) ;
 
-  /* now move the article into the third queue */
-  cxn->takeHead = cxn->checkRespHead ;
-  cxn->checkRespHead = NULL ;
-
-  issueIHAVEBody (cxn) ;
+  if (cxn->checkRespHead == NULL)
+    {
+      syslog (LOG_NOTICE,BAD_RESPONSE,
+              hostPeerName (cxn->myHost),cxn->ident,335) ;
+      cxnSleepOrDie (cxn) ;
+    }
+  else 
+    {
+      /* now move the article into the third queue */
+      cxn->takeHead = cxn->checkRespHead ;
+      cxn->checkRespHead = NULL ;
+      
+      issueIHAVEBody (cxn) ;
+    }
 }
 
 
@@ -2492,7 +2560,7 @@ static void noSuchMessageId (Connection cxn, u_int responseCode,
   if (msgid == NULL || strlen (msgid) == 0)
     syslog (LOG_ERR, NOMSGID, peerName, cxn->ident, responseCode, response) ;
   else
-    syslog (LOG_ERR, INVALID_MSGID, peerName, cxn->ident, msgid, responseCode) ;
+    syslog (LOG_ERR, INVALID_MSGID, peerName, cxn->ident, responseCode, msgid) ;
 
   cxnLogStats (cxn,true) ;
 
@@ -2581,6 +2649,7 @@ static void resetConnection (Connection cxn)
 
   bufferSetDataSize (cxn->respBuffer,0) ;
 
+  cxn->loggedNoCr = false ;
   cxn->maxCheck = 1 ;
   cxn->immedRecon = false ;
   cxn->doesStreaming = false ;  /* who knows, next time around maybe... */
@@ -3527,8 +3596,6 @@ static void appendArtHolder (ArtHolder artH, ArtHolder *head, u_int *count)
  */
 static ArtHolder artHolderByMsgId (const char *msgid, ArtHolder head)
 {
-  ASSERT (head != NULL) ;
-
   while (head != NULL)
     {
       if (strcmp (msgid, artMsgId (head->article)) == 0)
@@ -3539,3 +3606,28 @@ static ArtHolder artHolderByMsgId (const char *msgid, ArtHolder head)
 
   return NULL ;
 }
+
+
+
+/*
+ * Randomize a numeber by the given percentage
+ */
+
+static int fudgeFactor (int initVal)
+{
+  int newValue ;
+  static bool seeded ;
+
+  if ( !seeded )
+    {
+      time_t t = theTime () ;
+
+      /* this may have been done already in endpoint.c. Is that a problem??? */
+      srand (t) ;               
+      seeded = true ;
+    }
+  
+  newValue = initVal + (initVal / 10 - (rand() % (initVal / 5)));
+
+  return newValue ;
+}
