@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 17:10:18 1995
  * Project:     INN (innfeed)
  * File:        misc.c
- * RCSId:       $Id: misc.c,v 1.10 1996/05/23 21:27:32 brister Exp $
+ * RCSId:       $Id: misc.c,v 1.10 1996/05/23 21:27:32 brister Exp brister $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: misc.c,v 1.10 1996/05/23 21:27:32 brister Exp $" ;
+static const char *rcsid = "$Id: misc.c,v 1.10 1996/05/23 21:27:32 brister Exp brister $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -599,7 +599,7 @@ bool lockFile (const char *fileName)
                it's still alive. */
             if (kill (pid,0) == 0)
               {
-                syslog (LOG_ERR,LOCK_EXISTS,realName) ;
+                syslog (LOG_ERR,LOCK_EXISTS,realName,(int) pid) ;
                 unlink (tmpName) ;
                 return false ;    /* process is still alive */
               }
